Type: Package
Package: flextable
Title: Functions for Tabular Reporting
Version: 0.10.0
Authors@R: c(
    person("David", "Gohel", , "david.gohel@ardata.fr", role = c("aut", "cre")),
    person("ArData", role = "cph"),
    person("Clementine", "Jager", role = "ctb"),
    person("Eli", "Daniels", role = "ctb"),
    person("Panagiotis", "Skintzos", , "panagiotis.skintzos@ardata.fr", role = "aut"),
    person("Quentin", "Fazilleau", role = "ctb"),
    person("Maxim", "Nazarov", role = "ctb"),
    person("Titouan", "Robert", role = "ctb"),
    person("Michael", "Barrowman", role = "ctb"),
    person("Atsushi", "Yasumoto", role = "ctb"),
    person("Paul", "Julian", role = "ctb"),
    person("Sean", "Browning", role = "ctb"),
    person("Rémi", "Thériault", role = "ctb",
           comment = c(ORCID = "0000-0003-4315-6788")),
    person("Samuel", "Jobert", role = "ctb"),
    person("Keith", "Newman", role = "ctb")
  )
Description: Use a grammar for creating and customizing pretty tables.
    The following formats are supported: 'HTML', 'PDF', 'Typst', 'RTF',
    'Microsoft Word', 'Microsoft PowerPoint', R 'Grid Graphics' and
    'patchwork'.  'R Markdown', 'Quarto' and the package 'officer' can be
    used to produce the result files. The syntax is the same for the user
    regardless of the type of output to be produced. A set of functions
    allows the creation, definition of cell arrangement, addition of
    headers or footers, formatting and definition of cell content with
    text and or images. The package also offers a set of high-level
    functions that allow tabular reporting of statistical models and the
    creation of complex cross tabulations.
License: GPL-3
URL: https://ardata-fr.github.io/flextable-book/,
        https://davidgohel.github.io/flextable/
BugReports: https://github.com/davidgohel/flextable/issues
Imports: data.table (>= 1.13.0), gdtools (>= 0.5.0), graphics,
        grDevices, grid, htmltools, knitr, officer (>= 0.7.5), ragg,
        rlang, rmarkdown (>= 2.0), stats, utils, uuid (>= 0.1-4), xml2
Suggests: bookdown (>= 0.40), broom, broom.mixed, chromote, cluster,
        commonmark, doconv (>= 0.3.0), equatags, formatters, ggplot2,
        gtable, jsonlite, lme4, magick, mgcv, nlme, officedown,
        patchwork, pdftools, pkgdown (>= 2.0.0), rtables, scales,
        svglite, tables (>= 0.9.17), testthat (>= 3.0.0), webshot2,
        withr, xtable
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-07-07 08:49:11 UTC; davidgohel
Author: David Gohel [aut, cre],
  ArData [cph],
  Clementine Jager [ctb],
  Eli Daniels [ctb],
  Panagiotis Skintzos [aut],
  Quentin Fazilleau [ctb],
  Maxim Nazarov [ctb],
  Titouan Robert [ctb],
  Michael Barrowman [ctb],
  Atsushi Yasumoto [ctb],
  Paul Julian [ctb],
  Sean Browning [ctb],
  Rémi Thériault [ctb] (ORCID: <https://orcid.org/0000-0003-4315-6788>),
  Samuel Jobert [ctb],
  Keith Newman [ctb]
Maintainer: David Gohel <david.gohel@ardata.fr>
Repository: CRAN
Date/Publication: 2026-07-07 11:00:02 UTC
