# Generated by roxygen2: do not edit by hand

S3method(as_flextable,ElementaryTable)
S3method(as_flextable,TableTree)
S3method(as_flextable,brmsfit)
S3method(as_flextable,compact_summary)
S3method(as_flextable,data.frame)
S3method(as_flextable,gam)
S3method(as_flextable,glm)
S3method(as_flextable,glmmTMB)
S3method(as_flextable,glmmadmb)
S3method(as_flextable,gls)
S3method(as_flextable,grouped_data)
S3method(as_flextable,htest)
S3method(as_flextable,kmeans)
S3method(as_flextable,lm)
S3method(as_flextable,lme)
S3method(as_flextable,merMod)
S3method(as_flextable,nlme)
S3method(as_flextable,pam)
S3method(as_flextable,summarizor)
S3method(as_flextable,table)
S3method(as_flextable,tabular)
S3method(as_flextable,tabulator)
S3method(as_flextable,xtable)
S3method(dim,flextable)
S3method(dim,flextableGrob)
S3method(drawDetails,flextableHighlightedGrob)
S3method(drawDetails,flextableRowPartGrob)
S3method(format_fun,Date)
S3method(format_fun,POSIXt)
S3method(format_fun,character)
S3method(format_fun,default)
S3method(format_fun,double)
S3method(format_fun,factor)
S3method(format_fun,integer)
S3method(format_fun,logical)
S3method(format_fun,pct)
S3method(knit_print,flextable)
S3method(knit_print,run_reference)
S3method(latex_container_str,latex_container_float)
S3method(latex_container_str,latex_container_none)
S3method(latex_container_str,latex_container_wrap)
S3method(makeContext,flextableGrob)
S3method(ph_with,flextable)
S3method(plot,flextable)
S3method(plot,flextableGrob)
S3method(print,flextable)
S3method(print,flextable_defaults)
S3method(print,tabulator)
S3method(rtf_add,flextable)
S3method(summary,tabulator)
S3method(to_html,flextable)
S3method(to_rtf,flextable)
S3method(to_wml,flextable)
export(add_body)
export(add_body_row)
export(add_footer)
export(add_footer_lines)
export(add_footer_row)
export(add_header)
export(add_header_lines)
export(add_header_row)
export(add_latex_dep)
export(align)
export(align_nottext_col)
export(align_text_col)
export(append_chunks)
export(as_b)
export(as_bracket)
export(as_chunk)
export(as_equation)
export(as_flextable)
export(as_grouped_data)
export(as_highlight)
export(as_i)
export(as_image)
export(as_paragraph)
export(as_qmd)
export(as_raster)
export(as_strike)
export(as_sub)
export(as_sup)
export(as_word_field)
export(autofit)
export(before)
export(bg)
export(body_add_flextable)
export(body_replace_flextable_at_bkm)
export(bold)
export(border)
export(border_inner)
export(border_inner_h)
export(border_inner_v)
export(border_outer)
export(border_remove)
export(chunk_dataframe)
export(colformat_char)
export(colformat_date)
export(colformat_datetime)
export(colformat_double)
export(colformat_image)
export(colformat_int)
export(colformat_lgl)
export(colformat_num)
export(color)
export(colorize)
export(compact_summary)
export(compose)
export(continuous_summary)
export(delete_columns)
export(delete_part)
export(delete_rows)
export(df_printer)
export(dim_pretty)
export(empty_blanks)
export(fit_columns)
export(fit_to_width)
export(fix_border_issues)
export(flextable)
export(flextable_dim)
export(flextable_html_dependency)
export(flextable_to_rmd)
export(fmt_2stats)
export(fmt_avg_dev)
export(fmt_dbl)
export(fmt_header_n)
export(fmt_int)
export(fmt_n_percent)
export(fmt_pct)
export(fmt_signif_after_zeros)
export(fmt_summarizor)
export(font)
export(fontsize)
export(footers_flextable_at_bkm)
export(footnote)
export(fp_border_default)
export(fp_text_default)
export(gen_grob)
export(get_flextable_defaults)
export(gg_chunk)
export(grid_chunk)
export(headers_flextable_at_bkm)
export(height)
export(height_all)
export(highlight)
export(hline)
export(hline_bottom)
export(hline_top)
export(hrule)
export(htmltools_value)
export(hyperlink_text)
export(information_data_cell)
export(information_data_chunk)
export(information_data_paragraph)
export(init_flextable_defaults)
export(italic)
export(keep_with_next)
export(labelizor)
export(line_spacing)
export(linerange)
export(lollipop)
export(merge_at)
export(merge_h)
export(merge_h_range)
export(merge_none)
export(merge_v)
export(minibar)
export(mk_par)
export(ncol_keys)
export(nrow_part)
export(padding)
export(paginate)
export(plot_chunk)
export(prepend_chunks)
export(proc_freq)
export(qflextable)
export(regulartable)
export(rotate)
export(save_as_docx)
export(save_as_html)
export(save_as_image)
export(save_as_pptx)
export(save_as_rtf)
export(separate_header)
export(set_caption)
export(set_flextable_defaults)
export(set_footer_df)
export(set_formatter)
export(set_formatter_type)
export(set_header_df)
export(set_header_labels)
export(set_table_properties)
export(shift_table)
export(split_columns)
export(split_rows)
export(split_to_pages)
export(style)
export(summarizor)
export(surround)
export(tab_settings)
export(tabulator)
export(tabulator_colnames)
export(theme_alafoli)
export(theme_apa)
export(theme_booktabs)
export(theme_borderless)
export(theme_box)
export(theme_tron)
export(theme_tron_legacy)
export(theme_vader)
export(theme_vanilla)
export(theme_zebra)
export(use_df_printer)
export(use_flextable_qmd)
export(use_model_printer)
export(valign)
export(vline)
export(vline_left)
export(vline_right)
export(void)
export(width)
export(wrap_flextable)
import(data.table)
importFrom(data.table,":=")
importFrom(data.table,.GRP)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,fcase)
importFrom(data.table,fifelse)
importFrom(data.table,first)
importFrom(data.table,fsetdiff)
importFrom(data.table,groupingsets)
importFrom(data.table,is.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,rleid)
importFrom(data.table,rleidv)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setcolorder)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(data.table,shift)
importFrom(data.table,tstrsplit)
importFrom(gdtools,installed_gfonts)
importFrom(gdtools,strings_sizes)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,is.raster)
importFrom(grDevices,rgb)
importFrom(graphics,boxplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,drawDetails)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.layout)
importFrom(grid,grid.newpage)
importFrom(grid,grid.rect)
importFrom(grid,grobDescent)
importFrom(grid,grobHeight)
importFrom(grid,grobWidth)
importFrom(grid,is.grob)
importFrom(grid,makeContext)
importFrom(grid,popViewport)
importFrom(grid,pushViewport)
importFrom(grid,rasterGrob)
importFrom(grid,rectGrob)
importFrom(grid,segmentsGrob)
importFrom(grid,setChildren)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,viewport)
importFrom(grid,vpStack)
importFrom(htmltools,HTML)
importFrom(htmltools,attachDependencies)
importFrom(htmltools,browsable)
importFrom(htmltools,div)
importFrom(htmltools,htmlDependency)
importFrom(htmltools,htmlEscape)
importFrom(htmltools,save_html)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(knitr,asis_output)
importFrom(knitr,fig_path)
importFrom(knitr,include_graphics)
importFrom(knitr,is_html_output)
importFrom(knitr,is_latex_output)
importFrom(knitr,knit_child)
importFrom(knitr,knit_meta_add)
importFrom(knitr,knit_print)
importFrom(knitr,opts_current)
importFrom(knitr,opts_knit)
importFrom(knitr,pandoc_to)
importFrom(knitr,raw_block)
importFrom(knitr,raw_html)
importFrom(knitr,raw_latex)
importFrom(officer,add_slide)
importFrom(officer,body_add_par)
importFrom(officer,body_add_xml)
importFrom(officer,body_set_default_section)
importFrom(officer,cursor_bookmark)
importFrom(officer,fp_border)
importFrom(officer,fp_cell)
importFrom(officer,fp_par)
importFrom(officer,fp_par_lite)
importFrom(officer,fp_tab)
importFrom(officer,fp_tabs)
importFrom(officer,fp_text)
importFrom(officer,fp_text_lite)
importFrom(officer,fpar)
importFrom(officer,ftext)
importFrom(officer,image_to_base64)
importFrom(officer,officer_url_encode)
importFrom(officer,opts_current_table)
importFrom(officer,page_mar)
importFrom(officer,page_size)
importFrom(officer,ph_location_type)
importFrom(officer,ph_with)
importFrom(officer,prop_section)
importFrom(officer,prop_table)
importFrom(officer,read_docx)
importFrom(officer,read_pptx)
importFrom(officer,rtf_add)
importFrom(officer,rtf_doc)
importFrom(officer,run_autonum)
importFrom(officer,run_bookmark)
importFrom(officer,str_encode_to_rtf)
importFrom(officer,table_colwidths)
importFrom(officer,table_layout)
importFrom(officer,table_width)
importFrom(officer,to_html)
importFrom(officer,to_rtf)
importFrom(officer,to_wml)
importFrom(ragg,agg_capture)
importFrom(ragg,agg_png)
importFrom(rlang,"%||%")
importFrom(rlang,call_args)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,eval_tidy)
importFrom(rlang,get_expr)
importFrom(rlang,new_function)
importFrom(rlang,pairlist2)
importFrom(rlang,quo)
importFrom(rlang,quo_name)
importFrom(rlang,quo_text)
importFrom(rlang,sym)
importFrom(rmarkdown,html_document)
importFrom(rmarkdown,latex_dependency)
importFrom(rmarkdown,pandoc_available)
importFrom(rmarkdown,pandoc_version)
importFrom(rmarkdown,pdf_document)
importFrom(rmarkdown,render)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,IQR)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,density)
importFrom(stats,df.residual)
importFrom(stats,logLik)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,naprint)
importFrom(stats,nobs)
importFrom(stats,quantile)
importFrom(stats,reshape)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(utils,browseURL)
importFrom(utils,compareVersion)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(uuid,UUIDgenerate)
importFrom(xml2,as_xml_document)
