#' @title flextable: Functions for Tabular Reporting
#'
#' @description
#' The flextable package facilitates access to and manipulation of
#' tabular reporting elements from R.
#'
#' The documentation of functions can be opened with command `help(package = "flextable")`.
#'
#' To learn more about flextable, start with the vignettes:
#' `browseVignettes(package = "flextable")`.
#'
#' `flextable()` function is producing flexible tables where each cell
#' can contain several chunks of text with their own set of formatting
#' properties (bold, font color, etc.). Function [compose()] lets customise
#' text of cells.
#'
#' @seealso <https://davidgohel.github.io/flextable/>, [flextable()]
#' @docType package
#' @aliases flextable-package
#' @name flextable-package
NULL
