list(
  rd_family_title = list(
    cell_content_composition = "Other functions to compose cell content: ",
    table_structure = "Other row and column operations: ",
    flextable_constructors = "Other flextable constructors: ",
    flextable_configuration = "Other flextable configuration: ",
    formatting_shortcuts = "Other formatting shortcuts: ",
    default_formatting_properties = "Other default formatting properties:",
    flextable_output_export = "Other functions for flextable output and export: ",
    officer_integration = "Other functions for officer integration: ",
    themes_and_defaults = "Other themes and defaults: "
    )
)
