---
title: "Using flextable"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using flextable}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

The user manual is available here: 

https://ardata-fr.github.io/flextable-book/

Help pages of functions are available here: 

https://davidgohel.github.io/flextable/reference/index.html

