---
title: "use printers"
---

```{r setup, include=FALSE}
library(knitr)
opts_chunk$set(echo = FALSE)
library(flextable)
use_model_printer()
use_df_printer()
```

# a model

```{r}
clotting <- data.frame(
    u = c(5,10,15,20,30,40,60,80,100),
    lot1 = c(118,58,42,35,27,25,21,19,18),
    lot2 = c(69,35,26,21,18,16,13,12,12))
model <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
model
```

# no stars

```{r}
options(show.signif.stars = FALSE)
model
```

## example 2

```{r}
airquality
```
