% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizor.R
\name{fmt_pct}
\alias{fmt_pct}
\title{Format numbers as percentages}
\usage{
fmt_pct(x)
}
\arguments{
\item{x}{numeric values}
}
\description{
Format numeric values as percentages (e.g. \code{"45.0\%"}).
}
\examples{
library(flextable)

df <- data.frame(zz = .45)

ft_1 <- flextable(df)
ft_1 <- mk_par(
  x = ft_1, j = 1, part = "body",
  value = as_paragraph(as_chunk(zz, formatter = fmt_pct))
)
ft_1 <- autofit(ft_1)
ft_1
}
\seealso{
\code{\link[=tabulator]{tabulator()}}, \code{\link[=mk_par]{mk_par()}}

Other value formatters:
\code{\link[=fmt_2stats]{fmt_2stats()}},
\code{\link[=fmt_avg_dev]{fmt_avg_dev()}},
\code{\link[=fmt_dbl]{fmt_dbl()}},
\code{\link[=fmt_header_n]{fmt_header_n()}},
\code{\link[=fmt_int]{fmt_int()}},
\code{\link[=fmt_n_percent]{fmt_n_percent()}},
\code{\link[=fmt_signif_after_zeros]{fmt_signif_after_zeros()}}
}
\concept{value formatters}
