% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zz-defunct.R
\name{lollipop}
\alias{lollipop}
\title{Mini lollipop chart chunk wrapper}
\usage{
lollipop(...)
}
\description{
Function is now defunct, use \code{\link[=gg_chunk]{gg_chunk()}} instead.
}
\keyword{internal}
