% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_flextable.R
\name{continuous_summary}
\alias{continuous_summary}
\title{continuous columns summary}
\usage{
continuous_summary(
  dat,
  columns = NULL,
  by = character(0),
  hide_grouplabel = TRUE,
  digits = 3
)
}
\arguments{
\item{dat}{a data.frame}

\item{columns}{continuous variables to be summarized. If NULL all
continuous variables are summarized.}

\item{by}{discrete variables to use as groups when summarizing.}

\item{hide_grouplabel}{if TRUE, group label will not be rendered, only
level/value will be rendered.}

\item{digits}{the desired number of digits after the decimal point}
}
\description{
create a data.frame summary for continuous variables
}
\examples{
ft_1 <- continuous_summary(iris, names(iris)[1:4], by = "Species",
  hide_grouplabel = FALSE)
ft_1
}
