% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizor.R
\name{fmt_header_n}
\alias{fmt_header_n}
\title{Format count as '(N=XX)' for column headers}
\usage{
fmt_header_n(n, newline = TRUE)
}
\arguments{
\item{n}{count values}

\item{newline}{indicates to prefix the text with a new line
(sof return).}
}
\description{
Format counts as \verb{\\n(N=XX)} for appending
sample sizes to column titles.
}
\examples{
library(flextable)

df <- data.frame(zz = 1)

ft_1 <- flextable(df)
ft_1 <- append_chunks(
  x = ft_1, j = 1, part = "header",
  value = as_chunk(fmt_header_n(200))
)
ft_1 <- autofit(ft_1)
ft_1
}
\seealso{
\code{\link[=tabulator]{tabulator()}}, \code{\link[=mk_par]{mk_par()}}

Other value formatters:
\code{\link[=fmt_2stats]{fmt_2stats()}},
\code{\link[=fmt_avg_dev]{fmt_avg_dev()}},
\code{\link[=fmt_dbl]{fmt_dbl()}},
\code{\link[=fmt_int]{fmt_int()}},
\code{\link[=fmt_n_percent]{fmt_n_percent()}},
\code{\link[=fmt_pct]{fmt_pct()}},
\code{\link[=fmt_signif_after_zeros]{fmt_signif_after_zeros()}}
}
\concept{value formatters}
