% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc-selectors.R
\name{args_x_i_part_no_all}
\alias{args_x_i_part_no_all}
\title{internal utils for roxygen tags reuse}
\arguments{
\item{x}{a 'flextable' object, see \link{flextable-package} to learn how to create
'flextable' object.}

\item{i}{row selector, see section \emph{Row selection with the \code{i} parameter}
in <\code{\link[=flextable_selectors]{Selectors in flextable}}>.}

\item{part}{part selector, see section \emph{Part selection with the \code{part}
parameter} in <\code{\link[=flextable_selectors]{Selectors in flextable}}>.
Value 'all' is not allowed by the function.}
}
\description{
internal utils for roxygen tags reuse
}
\keyword{internal}
