% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_flextable.R
\name{merge_none}
\alias{merge_none}
\title{Delete flextable merging information}
\usage{
merge_none(x, part = "all")
}
\arguments{
\item{x}{a 'flextable' object, see \link{flextable-package} to learn how to create
'flextable' object.}

\item{part}{part selector, see section \emph{Part selection with the \code{part}
parameter} in <\code{\link[=flextable_selectors]{Selectors in flextable}}>.
Value 'all' can be used.}
}
\description{
Delete all merging information from a flextable.
}
\examples{
typology <- data.frame(
  col_keys = c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width", "Species"),
  what = c("Sepal", "Sepal", "Petal", "Petal", "Species"),
  measure = c("Length", "Width", "Length", "Width", "Species"),
  stringsAsFactors = FALSE
)

ft <- flextable(head(iris))
ft <- set_header_df(ft, mapping = typology, key = "col_keys")
ft <- merge_v(ft, j = c("Species"))

ft <- theme_tron_legacy(merge_none(ft))
ft
}
\seealso{
Other cell merging functions:
\code{\link[=merge_at]{merge_at()}},
\code{\link[=merge_h]{merge_h()}},
\code{\link[=merge_h_range]{merge_h_range()}},
\code{\link[=merge_v]{merge_v()}}
}
\concept{cell merging functions}
