% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latex_str.R
\name{add_latex_dep}
\alias{add_latex_dep}
\title{Add latex dependencies}
\usage{
add_latex_dep(float = FALSE, wrapfig = FALSE)
}
\arguments{
\item{float}{load package 'float'}

\item{wrapfig}{load package 'wrapfig'}
}
\description{
Manually add flextable latex dependencies to
the knitr session via \code{\link[knitr:knit_meta_add]{knitr::knit_meta_add()}}.

When enabling caching in 'R Markdown' documents for PDF output,
the flextable cached result is used directly. Call \code{add_latex_dep()} in a
non cached chunk so that flextable latex dependencies are added
to knitr metadata.
}
\examples{
add_latex_dep()
}
\seealso{
\code{\link[=knit_print.flextable]{knit_print.flextable()}}

Other flextable configuration:
\code{\link[=use_df_printer]{use_df_printer()}},
\code{\link[=use_flextable_qmd]{use_flextable_qmd()}},
\code{\link[=use_model_printer]{use_model_printer()}}
}
\concept{flextable_configuration}
\keyword{internal}
